#ifndef __NLL_IMAGE_H_INCLUDED__
#define __NLL_IMAGE_H_INCLUDED__

typedef unsigned int image_draw_count_t;

typedef enum {
  IMAGE_TYPE_NONE = -1,

  /* the index of image_procs[] */
  IMAGE_TYPE_FULLCOLOR = 0,
  IMAGE_TYPE_MONOCOLOR,
} image_type_t;

typedef enum {
  IMAGE_STATUS_NONE = 0,
  IMAGE_STATUS_ACTIVE,
  IMAGE_STATUS_FREE,
} image_status_t;

typedef struct image {
  int index;
  int width;
  int height;
  int size;
  int bytesize;
  image_type_t type;
  unsigned int flags;
#define IMAGE_FLAG_CONST  (1 << 16)
#define IMAGE_FLAG_STATIC (1 << 17)
  struct {
    int bg;
    int from;
    int to;
  } color;
  struct {
    int x;
    int y;
    int width;
    int height;
    int mulx;
    int muly;
  } range;
  struct {
    int x;
    int y;
  } offset;
  struct {
    struct {
      int x;
      int y;
    } mul;
    struct {
      int x;
      int y;
    } div;
  } scale;
  struct {
    int x;
    int y;
  } dotsize;
  struct {
    int x;
    int y;
    int color;
  } current;
  int *bitmap;
  volatile image_status_t status;
  struct image_draw {
    image_draw_count_t count;
    image_draw_count_t *line;
#define DRAW_COLUMN_PER 32
    int column_num;
    image_draw_count_t *column;
    image_draw_count_t *pixel;
  } draw;
  struct image_copy {
    int *bitmap;
  } copy;
} *image_t;

int image_init(void);
int image_done(void);
int image_check(void);
int image_is_draw_line(image_t image, image_draw_count_t count, int y);
int image_is_draw_column(image_t image, image_draw_count_t count, int x, int y);
int image_is_draw_pixel(image_t image, image_draw_count_t count, int x, int y);
int image_set_draw_line(image_t image, image_draw_count_t count, int y);
int image_set_draw_column(image_t image, image_draw_count_t count, int x, int y);
int image_set_draw_pixel(image_t image, image_draw_count_t count, int x, int y);
int image_get_color(image_t image, int x, int y);
int image_set_color(image_t image, int x, int y, int color, unsigned int flags);
int image_copy_save(image_t image, int x, int y, int width, int height);
int image_copy_get_color(image_t image, int x, int y);
int image_copy_save_color(image_t image, int x, int y);
int image_get_index(image_t image);
image_t image_get_image(int index);
int image_reset_draw(image_t image);
int image_destroy(image_t image);
int image_create(image_t image, int width, int height, int *bitmap,
		 image_type_t type, unsigned int flags);
image_t image_get(int width, int height, int *bitmap,
		  image_type_t type, unsigned int flags);

#endif
